
clear;

%% Load and set up data

load BAC_Application_BAC_Ret_Vol_Volume_Results.mat;

returns_table = bac_returns_volume_vol;

common_dates = intersect(returns_table.Properties.RowTimes,parameter_estimates.Properties.RowTimes);

double_dates = common_dates(arrayfun(@(x) sum(parameter_estimates.Properties.RowTimes == x), common_dates) > 1);

for i=1:size(double_dates, 1)
    j = find(parameter_estimates.Properties.RowTimes == double_dates(i));
    parameter_estimates(j(1), :) = [];
end

merged_data = [returns_table(common_dates,:) parameter_estimates(common_dates,:)];


%% Estimate censored likelihood scores (see eq. (10) in C. Diks et al. (2011))

r = -.05; % set return cutoff
a = 30; % logistic weight shape parameter

scores = array2timetable(NaN(size(merged_data,1)-1,4),'VariableNames',{'QMLEINDEP','QMLEGAUSS','QMLESTUDENT', 'SMLE'},...
    'RowTimes',merged_data.Properties.RowTimes(2:end));

for i=1:size(merged_data,1)-1
    
    disp(['Calculating scores for ' datestr(merged_data.Properties.RowTimes(i))]);
    
    % build marginal models using parameter estimates
    
    qmle_indep_model = makedist('tLocationScale',...
        'mu',merged_data{i,4}{1}.QMLEINDEP(1),...
        'sigma',merged_data{i,4}{1}.QMLEINDEP(2),...
        'nu',merged_data{i,4}{1}.QMLEINDEP(3));
    
    qmle_gauss_model = makedist('tLocationScale',...
        'mu',merged_data{i,4}{1}.QMLEGAUSS(1),...
        'sigma',merged_data{i,4}{1}.QMLEGAUSS(2),...
        'nu',merged_data{i,4}{1}.QMLEGAUSS(3));

    qmle_student_model = makedist('tLocationScale',...
        'mu',merged_data{i,4}{1}.QMLESTUDENT(1),...
        'sigma',merged_data{i,4}{1}.QMLESTUDENT(2),...
        'nu',merged_data{i,4}{1}.QMLESTUDENT(3));    
    
    qmle_smle_model = makedist('tLocationScale',...
        'mu',merged_data{i,4}{1}.SMLE(1),...
        'sigma',merged_data{i,4}{1}.SMLE(2),...
        'nu',merged_data{i,4}{1}.SMLE(3));
    
    % calculate scores
    
    y_tplusone = merged_data{i+1,1};
    w_t = (1/(1+exp(a*(y_tplusone-r))));
    
    % QMLE - Independence    
    
    fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_indep_model,x); % numeric part      
    
    scores{i,1} = w_t*log(pdf(qmle_indep_model,y_tplusone)) + ...
        (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score
    
    % QMLE - Gaussian copula
    
    fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_gauss_model,x); % numeric part    
    
    scores{i,2} = w_t*log(pdf(qmle_gauss_model,y_tplusone)) + ...
        (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score

    % QMLE - t-copula
    
    fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_student_model,x); % numeric part    
    
    scores{i,3} = w_t*log(pdf(qmle_student_model,y_tplusone)) + ...
        (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score    
    
    % QMLE - Sieve copula 
    
    fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_smle_model,x); % numeric part    
    
    scores{i,4} = w_t*log(pdf(qmle_smle_model,y_tplusone)) + ...
        (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score   
    
end

%% Look at mean scores and test for difference

mean(scores.Variables)
full_scores = scores;

%% Estimate standard errors for differences using delete-d Jack-Knife

% tr = timerange(merged_data.Properties.RowTimes(1),merged_data.Properties.RowTimes(end));
% tr = timerange(datetime('1-Jan-2006'),datetime('31-Dec-2012')); % crisis
% merged_data = merged_data(tr,:);

n = size(merged_data,1);
d = 5;
b = ceil(n/d); % number of bootstrap samples
deletion_index_matrix = reshape([1:b*d]',d,b);
deletion_index_matrix(deletion_index_matrix > n) = NaN(size(deletion_index_matrix(deletion_index_matrix > n)));

differences = NaN(b,2); % container for differences

parfor j=1:b
    
    % Generate delete-d jack-knife sample
    
    delete_index = deletion_index_matrix(:,j);
    delete_index = delete_index(~isnan(delete_index));
    
    tmpData = merged_data(~ismember([1:n],delete_index)',:);
    
    scores = array2timetable(NaN(size(tmpData,1)-1,4),'VariableNames',{'QMLEINDEP','QMLEGAUSS','QMLESTUDENT', 'SMLE'},...
        'RowTimes',tmpData.Properties.RowTimes(2:end));    
    
    for i=1:size(tmpData,1)-1
    
%         disp(['Calculating scores for ' datestr(tmpData.Properties.RowTimes(i))]);

        % build marginal models using parameter estimates

        qmle_indep_model = makedist('tLocationScale',...
            'mu',tmpData{i,4}{1}.QMLEINDEP(1),...
            'sigma',tmpData{i,4}{1}.QMLEINDEP(2),...
            'nu',tmpData{i,4}{1}.QMLEINDEP(3));

        qmle_student_model = makedist('tLocationScale',...
            'mu',tmpData{i,4}{1}.QMLESTUDENT(1),...
            'sigma',tmpData{i,4}{1}.QMLESTUDENT(2),...
            'nu',tmpData{i,4}{1}.QMLESTUDENT(3));

        qmle_gauss_model = makedist('tLocationScale',...
            'mu',tmpData{i,4}{1}.QMLEGAUSS(1),...
            'sigma',tmpData{i,4}{1}.QMLEGAUSS(2),...
            'nu',tmpData{i,4}{1}.QMLEGAUSS(3));        

        qmle_smle_model = makedist('tLocationScale',...
            'mu',tmpData{i,4}{1}.SMLE(1),...
            'sigma',tmpData{i,4}{1}.SMLE(2),...
            'nu',tmpData{i,4}{1}.SMLE(3));

        % calculate scores

        y_tplusone = tmpData{i+1,1};
        w_t = (1/(1+exp(a*(y_tplusone-r))));

        % QMLE - Independence    

        fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_indep_model,x); % numeric part      

        scores{i,1} = w_t*log(pdf(qmle_indep_model,y_tplusone)) + ...
            (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score

        % FMLE - Gaussian copula

        fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_gauss_model,x); % numeric part    

        scores{i,2} = w_t*log(pdf(qmle_gauss_model,y_tplusone)) + ...
            (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score

        % FMLE - t-copula

        fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_student_model,x); % numeric part    

        scores{i,3} = w_t*log(pdf(qmle_student_model,y_tplusone)) + ...
            (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score

        % QMLE - Sieve copula 

        fun = @(x) ((1/(1+exp(a*(x-r))))').*pdf(qmle_smle_model,x); % numeric part    

        scores{i,4} = w_t*log(pdf(qmle_smle_model,y_tplusone)) + ...
            (1-w_t)*integral(fun,-Inf,Inf,'ArrayValued',true); % observation score   
    
    end 
    
    score_means = mean(scores.Variables);
    differences(j,:) = [score_means(4) - score_means(1) ... % SMPLE - INDEP difference
        score_means(4) - score_means(3)]; %
    
    j
    
end

%% Estimate test statistics

% H0: SMLE yields higher score than QMLE-INDEP

jack_knife_variance = sqrt(((n-d)/(d*b)) * sum((differences(:,1) - sum(differences(:,1))/b).^2));
score_means = mean(full_scores.Variables);
test_statistic = (score_means(4) - score_means(1))/jack_knife_variance

% H0: SMLE yields higher score than QMLE-TCOPULA

jack_knife_variance = sqrt(((n-d)/(d*b)) * sum((differences(:,2) - sum(differences(:,2))/b).^2));
score_means = mean(full_scores.Variables);
test_statistic = (score_means(4) - score_means(3))/jack_knife_variance