clear;

load BAC.mat

%% Prep data

full_list_of_dates = BAC.Properties.RowTimes(1):BAC.Properties.RowTimes(end);

BAC = fillmissing(synchronize(BAC,...
    timetable(full_list_of_dates',NaN(length(full_list_of_dates),1))),'previous');

BAC(BAC.Volume == 0,:).Volume = NaN(length(BAC(BAC.Volume == 0,:).Volume),1);
BAC.Volume = fillmissing(BAC.Volume,'previous');

BAC = addvars(BAC,BAC.AdjClose.*BAC.Volume,'NewVariableNames',{'DollarVolume'});

% estimate weekly returns and associated vol

merged_data = timetable(BAC.Properties.RowTimes(2:end),...
    price2ret(BAC.AdjClose),BAC.DollarVolume(2:end),'VariableNames',{'BAC','Volume'}); % BAC dailies plus dollar volume

fridays = merged_data.Properties.RowTimes(weekday(merged_data.Properties.RowTimes) == 6);
bac_returns_volume_vol = array2timetable(NaN(length(fridays)-1,3),'RowTimes',fridays(2:end),...
    'VariableNames',{'RET','VOLUME', 'VOLATILITY'});

for i=2:length(fridays)
    
    i
    
    bac_returns_volume_vol{i-1,:} = ...
        [sum(merged_data(fridays(i-1):fridays(i),:).BAC) sum(merged_data(fridays(i-1):fridays(i),:).Volume) std(merged_data(fridays(i-1):fridays(i),:).BAC)*sqrt(5)];
    
end

bac_returns_volume_vol.VOLUME(2:end) = price2ret(bac_returns_volume_vol.VOLUME);
bac_returns_volume_vol = bac_returns_volume_vol(2:end,:);

bac_returns_volume_vol(bac_returns_volume_vol.VOLATILITY == 0,:).VOLATILITY  = ...
    NaN(length(bac_returns_volume_vol(bac_returns_volume_vol.VOLATILITY == 0,:).VOLATILITY),1);
bac_returns_volume_vol.VOLATILITY = fillmissing(bac_returns_volume_vol.VOLATILITY,'previous');

%% Set parameters

% Rolling window and trade dates

rolling_window_width = days(365*5); % in days

trade_dates = bac_returns_volume_vol.Properties.RowTimes(bac_returns_volume_vol.Properties.RowTimes > ...
    bac_returns_volume_vol.Properties.RowTimes(1) + rolling_window_width);

% Potentially truncate trade_dates to ensure simulation doesn't take too
% long

trade_dates = trade_dates(trade_dates >= datetime(2012, 05, 04));

% Placeholders for VaR estimates and exceedance counter

BAC_var_estimates = table(zeros(length(trade_dates),1),zeros(length(trade_dates),1),zeros(length(trade_dates),1),zeros(length(trade_dates),1),...
    'VariableNames',{'QMLEINDEP','QMLEGAUSS','QMLESTUDENT', 'SMLE'}); % first column - MLE, second SMLE

parameter_estimates = timetable(trade_dates,cell(length(trade_dates),1));

% exceedance counters

BAC_exceeds = BAC_var_estimates; 

%% Set optimization constraints

J_N = 5;

marginals_lb = [repmat([-Inf;eps;eps],2,1);eps;eps];
marginals_ub = [repmat([Inf;Inf;100],2,1);Inf;Inf];

copula_parameters_lb = zeros(J_N^3,1);
copula_parameters_ub = ones(J_N^3,1);

lb = [marginals_lb;copula_parameters_lb]; % lower bounds on model parameters
ub = [marginals_ub;copula_parameters_ub]; % upper bounds on model parameters

% constraints on Sieve parameters to ensure it's a copula

k_marg = 8; % number of parameters in marginal models

Aeq = [zeros(1,k_marg) ones(1,J_N^3)]; % require that all sieve parameters (omegas) sum to one

% set up equality constraints to ensure sieve parameters sum to one along
% all individual dimensions as well

parameter_index_matrix = reshape(1:J_N^3,J_N,J_N,J_N);
A_left = zeros(J_N*3, k_marg);
AA = [];

% First dimension

for i=1:J_N   
    x_cont = zeros(1, J_N^3);
    x_cont(reshape(parameter_index_matrix(i, :, :), 1, J_N^2)) = ones(1, J_N^2);
    AA = [AA; x_cont];
end

% Second dimension

for j=1:J_N   
    x_cont = zeros(1, J_N^3);
    x_cont(reshape(parameter_index_matrix(:, j, :), 1, J_N^2)) = ones(1, J_N^2);
    AA = [AA; x_cont];
end

% Third dimension

for k=1:J_N   
    x_cont = zeros(1, J_N^3);
    x_cont(reshape(parameter_index_matrix(:, :, k), 1, J_N^2)) = ones(1, J_N^2);
    AA = [AA; x_cont];
end

Aeq = [Aeq;[A_left AA]];
beq = [1;repmat(1/(J_N),3*(J_N),1)];

options = optimoptions('fmincon','Display','iter','MaxFunctionEvaluations',10000);

%% Estimate VaR Values

for i=1:length(trade_dates)
    
    disp(trade_dates(i));
    
    xhat = table(zeros(k_marg,1),zeros(k_marg,1),zeros(k_marg,1),'VariableNames',...
        {'QMLEINDEP','QMLEGAUSS','SMLE'},...
        'RowNames',{'mu1','sigma1','nu1','mu2','sigma2','nu2','a','b'});
    
    subdata = bac_returns_volume_vol(trade_dates(i)-rolling_window_width:trade_dates(i),:);
    
    bac_returns = subdata.RET;
    bac_volume = subdata.VOLUME;
    bac_volatility = subdata.VOLATILITY;
    
    % Estimate 5% VaR for SPY - QMLE - Independence
    
    pd = fitdist(bac_volatility,'Beta');
    
    x0 = [mean(bac_returns);std(bac_returns);10;... % marginal parameters starting values
        mean(bac_volume);std(bac_volume);10;...
        pd.a;pd.b];  
    
    [x,fval,exitflag,output] = fmincon(@(params) QMLELogLikRetVolVolume(params,[bac_returns bac_volume bac_volatility]),x0,[],[],[],[],lb(1:k_marg),ub(1:k_marg),[],options);
    
    disp(['QMLEINDEP exit flag: ' num2str(exitflag)]);

    BAC_returns_model = makedist('tLocationScale',...
    'mu',x(1),...
    'sigma',x(2),...
    'nu',x(3));

    xhat.QMLEINDEP = x(1:k_marg);

    BAC_var_estimates.QMLEINDEP(i) = icdf(BAC_returns_model,0.05);
    
    % Estimate 5% VaR for SPY - QMLE - Gaussian
     
    vcv = corr([bac_returns bac_volume bac_volatility]);
    
    x0 = [x0;vcv(1,2);vcv(1,3);vcv(2,3)];
    
    [x,fval,exitflag,output] = fmincon(@(params) QMLELogLikGaussianRetVolVolume(params,[bac_returns bac_volume bac_volatility]),x0,[],[],[],[],[lb(1:k_marg);repmat(-1,3,1)],[ub(1:k_marg);ones(3,1)],[],options);
    
    disp(['QMLEGAUSS exit flag: ' num2str(exitflag)]);

    BAC_Returns_Model = makedist('tLocationScale',...
    'mu',x(1),...
    'sigma',x(2),...
    'nu',x(3)); 

    xhat.QMLEGAUSS = x(1:k_marg);

    BAC_var_estimates.QMLEGAUSS(i) = icdf(BAC_Returns_Model,0.05);

   % Estimate 5% VaR for SPY - QMLE - t-Copula
     
    vcv = corr([bac_returns bac_volume bac_volatility]);
    
    x0 = [x0;12];
    
    [x,fval,exitflag,output] = fmincon(@(params) QMLELogLikStudentRetVolVolume(params,[bac_returns bac_volume bac_volatility]),x0,[],[],[],[],[lb(1:k_marg);repmat(-1,3,1);eps],[ub(1:k_marg);ones(3,1);Inf],[],options);
    
    disp(['QMLESTUDENT exit flag: ' num2str(exitflag)]);

    BAC_Returns_Model = makedist('tLocationScale',...
    'mu',x(1),...
    'sigma',x(2),...
    'nu',x(3)); 

    xhat.QMLESTUDENT = x(1:k_marg);

    BAC_var_estimates.QMLESTUDENT(i) = icdf(BAC_Returns_Model,0.05);    
    
    % Estimate 5% VaR for SPY - Sieve MLE
    
    u = tiedrank(bac_returns)/(length(bac_returns)+1);
    v = tiedrank(bac_volume)/(length(bac_volume)+1);
    z = tiedrank(bac_volatility)/(length(bac_volatility)+1);
    
     % Estimate starting point for MLE
    % set initial optimization values
    
    w0 = zeros(J_N,J_N,J_N);

    for v_1 = 0:J_N-1        
            for v_2 = 0:J_N-1
                for v_3 = 0:J_N-1
                    w0(v_1+1,v_2+1,v_3+1) = sum(...
                        ((v_1/J_N)<=u).*(u<=((v_1+1)/J_N)).*...
                        ((v_2/J_N)<=v).*(v<=((v_2+1)/J_N)).*...
                        ((v_3/J_N)<=z).*(z<=((v_3+1)/J_N)))/length(u);            
                end
            end
    end
   
    x0 = [mean(bac_returns);std(bac_returns);10;... % Returns model marginal parameters starting values
        mean(bac_volume);std(bac_volume);10;... % Volume changes marginal parameters starting values
        pd.a;pd.b;... % Volatility marginal parameters starting values
        reshape(w0,[J_N^3,1])];
    
    % SMLE
    tic
    [x,fval,exitflag,output] = fmincon(@(params) sieveLogLikRetVolVolume(params,[bac_returns bac_volume bac_volatility],J_N),x0,[],[],Aeq,beq,lb,ub,[],options);
    toc
    
    disp(['SIEVE MLE exit flag: ' num2str(exitflag)]);
    
    xhat.SMLE = x(1:k_marg);
    
    disp(xhat);
    
    BAC_Returns_Model = makedist('tLocationScale',...
        'mu',x(1),...
        'sigma',x(2),...
        'nu',x(3)); 
    
    BAC_var_estimates.SMLE(i) = icdf(BAC_Returns_Model,0.05);
  
    BAC_exceeds(i,:).Variables = BAC_var_estimates(i,:).Variables>bac_returns(end);

    disp(BAC_var_estimates(i,:));
    disp(BAC_exceeds(i,:));
    
    parameter_estimates.Var1{i} = xhat;
    
end

%% Save results

save BAC_Application_BAC_Ret_Vol_Volume_Results.mat